package MT::Plugin::Jaccessibility;
use strict;

require MT::Plugin;

@MT::Plugin::Jaccessibility::ISA = qw(MT::Plugin);

our $version = "0.6";

my $plugin = new MT::Plugin::Jaccessibility({
	name => 'Jaccessibility',
	version => $version,
	description => '<MT_TRANS phrase=\'_PLUGIN_DESCRIPTION\'>',
	author_name => 'Junnama Noda',
	author_link => 'http://junnama.alfasado.net/online/',
	l10n_class => 'Jaccessibility::L10N',
});

MT->add_plugin($plugin);

MT::Template::Context->add_global_filter(jaccessibility => \&_jaccessibility);
MT::Template::Context->add_global_filter(style2plain => \&_style2plain);
MT::Template::Context->add_global_filter(remove_spacer => \&_remove_spacer);
MT::Template::Context->add_global_filter(remove_js => \&_remove_js);
MT::Template::Context->add_global_filter(obj2link => \&_obj2link);

MT->add_callback('MT::App::CMS::AppTemplateSource.upload', 9, $plugin, \&_AddInputField);
MT->add_callback('MT::App::CMS::AppTemplateParam.show_upload_html', 8, $plugin,\&_ReplaceParam);
MT->add_callback('MT::App::CMS::AppTemplateParam.edit_entry', 8, $plugin,\&_ReplaceParam);
MT->add_callback('MT::App::CMS::AppTemplateOutput.upload_complete', 9, $plugin, \&_AddParam);
MT->add_callback('MT::App::CMS::AppTemplateSource.upload_confirm', 9, $plugin, \&_AddHidden);
MT->add_callback('MT::App::CMS::AppTemplateOutput.edit_entry', 9, $plugin, \&_TryGetAlt);

my ($kan_y, $kan_m, $kan_d) = ();
my ($kan_j, $kan_h, $kan_b) = ();

sub _jaccessibility {
	my($text, $arg, $ctx) = @_;
	my @plug_args;
	if($arg == 0){
		@plug_args = (1, 2, 3, 4);
	}else{
		@plug_args = split(/\,/,$arg);
	}
	use utf8;
	use Encode;
	use Encode::Guess qw/ utf8 euc-jp shiftjis 7bit-jis /;
	$text = decode('utf-8', $text);
	my $global_str = encode('utf-8', '年,月,日,時,分,秒');
	($kan_y, $kan_m, $kan_d, $kan_j, $kan_h, $kan_b) = split(/\,/,$global_str);
	if (grep(/1/,@plug_args)) {
		my $tild = encode('utf-8', '〜');
		$text =~ s/＜/&lt;/g;
		$text =~ s/＞/&gt;/g;
		$text =~ s/＆/&amp;/g;
 		$text =~ s/“/&quot;/g;
  		$text =~ s/”/&quot;/g;
		my $sep = '&#124;';
		$text =~ s/\|/$sep/g;
		use Unicode::Normalize;
		my $tilde = '&#126;';
		$text =~ s/~/$tilde/g;
		$text = NFKC($text);
		$text =~ s/~/$tild/g;
	}
	if (grep(/2/,@plug_args)) {
		my $flag = 1;
		while ($flag == 1) {
			my $old = $text;
			$text =~ s/(\p{Ja})\s{1,}(\p{Ja})/$1$2/g;
			if ($old eq $text) {
				$flag = 0;
			}
		}
	}
	$text = encode('utf-8', $text);
	if (grep(/3/,@plug_args)) {
		my $local_str = ('円,ドル,ユーロ,セント,ポンド');
		$text =~ s/Â¥/\\/g;
		my $sep = '&#124;';
		$text =~ s/\|/$sep/g;
		$text = decode('utf-8', $text);
		$text =~ s/([$|€|£|¢|\\]|\&#36;|\&euro;|\&cent;|\&pound;|\&yen;)([0-9|\,]{1,})/&currency($1,$2,$local_str)/eg;
		$text = encode('utf-8', $text);
	}
	if (grep(/4/,@plug_args)) {
		my $local_str = encode('utf-8', '平成,昭和');
		my ($heisei, $syouwa) = split(/\,/,$local_str);
		$text =~ s/H([0-9]{2})[-\/\.]([0-9]{1,2})[-\/\.]([0-9]{1,2})/$heisei$1$kan_y$2$kan_m$3$kan_d/g;
		$text =~ s/H([0-9]{2})\.(\s[0-9]{1,2})\.(\s[0-9]{1,2})/$heisei$1$kan_y$2$kan_m$3$kan_d/g;
		$text =~ s/S([0-9]{2})[-\/\.]([0-9]{1,2})[-\/\.]([0-9]{1,2})/$syouwa$1$kan_y$2$kan_m$3$kan_d/g;
		$text =~ s/S([0-9]{2})\.(\s[0-9]{1,2})\.(\s[0-9]{1,2})/$syouwa$1$kan_y$2$kan_m$3$kan_d/g;
		my $S = '-';
		$text =~ s/(?<![\/|"])([0-9]{4})-([0-9]{1,2})-([0-9]{1,2})/&checkdate($1,$2,$3,$S)/eg;
		$S = '/';
		$text =~ s/(?<![\/|"])([0-9]{4})\/([0-9]{1,2})\/([0-9]{1,2})/&checkdate($1,$2,$3,$S)/eg;
		$S = '.';
		$text =~ s/(?<![\/|"])([0-9]{4})\.([0-9]{1,2})\.([0-9]{1,2})/&checkdate($1,$2,$3,$S)/eg;
		$text =~ s/(?<![\/|"])([0-9]{4}):([0-9]{1,2}):([0-9]{1,2})\s\+[0-9]{4}/&checktime($1,$2,$3,':')/g;
		$text =~ s/(?<![\/|"])([0-9]{4}):([0-9]{1,2}):([0-9]{1,2})/&checktime($1,$2,$3,':')/g;
		$text =~ s/(?<![\/|"])([A-Za-z]{3}),\s([0-9]{2})\s([A-Za-z]{3})\s([0-9]{4})/monthToInt($4,$3,$2,$1,$kan_y,$kan_m,$kan_d)/eg;
	}
	return $text;
}

sub _style2plain{
	my($text, $arg, $ctx) = @_;
	$text =~ s/<link.*rel=?("|')stylesheet?("|').*?>//isg;
	$text =~ s/<style.{1,}?<\/style>//isg;
	$text =~ s/<[\/]*(font|blink|marquee)[^>]*?>//isg;
	$text =~ s/(<[\/]*?)(center?)([\s]*?>)/$1div$3/isg;
	$text =~ s/(<[\/]*?)(b?)([\s]*?>)/$1strong$3/isg;
	$text =~ s/(<[\/]*?)(i?)([\s]*?>)/$1em$3/isg;
	$text =~ s/(<[\/]*?)(s?)([\s]*?>)/$1del$3/isg;
	$text =~ s/(<[\/]*?)(strike?)([\s]*?>)/$1del$3/isg;
	$text =~ s/<[\/]*u\s*?>//isg;
	$text =~ s/(<[^>]*)(style|bgcolor|align|vspace|hspace|background|text|link|vlink|alink|clear)=?("|')[^>]*?("|')/$1/isg;
	$text =~ s/\s*?>/>/isg;
	return $text;
}

sub _remove_js {
	my($text, $arg, $ctx) = @_;
	$text =~ s/<script.{1,}?<\/script>//isg;
	$text =~ s/(<[^>]*)(onblur|onchange|onclick|ondblclick|onfocus|onkeydown)=?("|')[^>]*?("|')/$1/isg;
	$text =~ s/(<[^>]*)(onkeypress|onkeyup|onload|onmousedown|onmousemove|onmouseout)=?("|')[^>]*?("|')/$1/isg;	
	$text =~ s/(<[^>]*)(onmouseover|onmouseup|onreset|onselect|onsubmit|onunload)=?("|')[^>]*?("|')/$1/isg;			
	$text =~ s/<a[^>]*href=?("|')javascript:.[^>]*?("|').*?>.*?<\/a>//isg;
	$text =~ s/<[\/]*noscript[^>]*?>//isg;
	return $text;
}

sub _remove_spacer {
	my($text, $arg, $ctx) = @_;
	#my $arg = 'spacer.gif';
	$text =~ s/<img[^>]*src=*["'][^"'>]*$arg["'][^>]*>//isg;
	return $text;
}

sub _obj2link {
	my($text, $arg, $ctx) = @_;
	my @plug_args;
	if($arg == 0){
		@plug_args = (1, 2, 3, 4);
	}else{
		@plug_args = split(/\,/,$arg);
	}
	if (grep(/1/,@plug_args)) {
		my $img = $plugin->translate('Image');
		my $lnk = $plugin->translate('Link');
		$text =~ s/(<a[^>].{1,}?>)<img[^>]*src=["']([^"'>]*)["'][^>]*alt=\n*["']([^"'>]*)["'][^>]*><\/a>/[<a href="$2">$img:$3<\/a>][$1$lnk<\/a>]/ig;
		$text =~ s/(<a[^>].{1,}?>)<img[^>]*alt=["']([^"'>]*)["'][^>]*src=\n*["']([^"'>]*)["'][^>]*><\/a>/[<a href="$3">$img:$2<\/a>][$1$lnk<\/a>]/ig;
		$text =~ s/<img[^>]*src=*["']([^"'>]*)["'][^>]*alt=\n*["']([^"'>]*)["'][^>]*>/[<a href="$1">$img:$2<\/a>]/ig;
		$text =~ s/<img[^>]*alt=*["']([^"'>]*)["'][^>]*src=\n*["']([^"'>]*)["'][^>]*>/[<a href="$2">$img:$1<\/a>]/ig;
		$text =~ s/(<a[^>].{1,}>)<img[^>]*src=*["']([^"'>]*)["'][^>]*><\/a>/[<a href="$2">$img<\/a>][$1\$lnk<\/a>]/ig;
		$text =~ s/<img[^>]*src=*["']([^"'>]*)["'][^>]*>/[<a href="$1">$img<\/a>]/ig;
		$text =~ s/(\[<a\shref="\/.{1,}?">$img.{1,}?<\/a>\])\[<a\shref="\/.{1,}?">$lnk<\/a>\]/$1/isg;
	}
	if (grep(/2/,@plug_args)) {
		my $frame = $plugin->translate('Frame');
		$text =~ s/<iframe[^>]*src=*["']([^"'>]*)["'][^>]*>*?<\/iframe>/[<a href="$1">$frame$2<\/a>]/ig;
	}
	if (grep(/3/,@plug_args)) {
		my $embed = $plugin->translate('Embed');
		$text =~ s/<embed[^>]*src=*["']([^"'>]*)["'][^>]*>*?<\/embed>/[<a href="$1">$embed$2<\/a>]/ig;
	}
	if (grep(/4/,@plug_args)) {
		my $cite = $plugin->translate('Cite');
		$text =~ s/<blockquote[^>]*cite="(.{1,}?)"[^>]*>(.{1,}?)<\/blockquote>/[<a href="$1">$cite<\/a>]$2/isg;
	}
	return $text;
}

sub currency {
	my ($curr, $int, $local_str) = @_;
	my ($yen, $daller, $euro, $cent, $pound) = split(/\,/,$local_str);
	if (($curr eq '\\') || ($curr eq '&yen;')) {
		return $int.$yen;
	} elsif (($curr eq '$') || ('&#36;')){
		return $int.$daller;
	} elsif (($curr eq '€') || ($curr eq '&euro;')) {
		return $int.$euro;
	} elsif (($curr eq '¢') || ($curr eq '&cent;')) {
		return $int.$cent;
	} elsif (($curr eq '£') || ($curr eq '&pound;')) {
		return $int.$pound;
	} else {
		return $int.$yen;
	}
}

sub _ReplaceParam {
	my ($eh, $app, $param_ref, $tmpl) = @_;
	my $search = $app->param;
	my $alt = $search->param('altField');
	my $url = $search->param('url');
	my $thumb = $search->param('thumb');
	my $blogId = $search->param('blog_id');
	$alt =~s/\n//g;
	my $textrea;
	if ($search->param('__mode') eq 'start_upload_entry') {
		$textrea = 'text';
	}else{
		$textrea = 'upload_html';
	}
	my $replace_str = $param_ref->{$textrea};
	$param_ref->{'url'} = $url;
	if (defined($replace_str)) {
		if (substr($replace_str,0,5) eq '<img '){
			$replace_str =~s/<img\salt=".{1,}?"/<img alt=\"$alt\"/;
			$replace_str =~s/\ssrc=(".{1,}?")/ src=\"$url\"/;
			$param_ref->{$textrea} = $replace_str;
		}else{
			if ($thumb != 1) {
				$replace_str =~s/>.{1,}</>$alt</;
			}
			$replace_str =~s/<img\salt=".{1,}?"/<img alt=\"$alt\"/;
			$param_ref->{$textrea} = $replace_str;
		}
	}
1;
}

sub _AddInputField {
	my ($eh, $app, $tmpl_ref) = @_;	
	my $old = '<input type="file" name="file" /></p>';
	my $label = $plugin->translate('Alt text in img (link text) : ');
	my $new = $old.'<p>'.$label.'<input type="text" name="altField" id="altField" /></p>';
	$old = quotemeta($old);
	$$tmpl_ref =~ s/$old/$new/;
	$old = quotemeta('<input type="submit"');
	$new = '<input onclick="return altCheck()" type="submit"';
	$$tmpl_ref =~ s/$old/$new/;
	my $msg = $plugin->translate('Required attribute "alt" or link text not specified.\\nIs uploading continued?');
	my $jscode = <<"END_OF_JS";
</form>
<script type="text/javascript">
tryGetAlt();
function tryGetAlt(){
	var str = window.opener.getAltText();
	if (str.indexOf("\\n") != -1){
		str = str.replace(/\\n/g,"");
	}
	document.getElementById("altField").value = str;
}
function altCheck(){
	var altTxt = document.getElementById("altField").value;
	if (altTxt == ""){
		return confirm("$msg");
	}
}
</script>
END_OF_JS
	$$tmpl_ref =~ s/<\/form>/$jscode<\/form>/;
1;
}

sub _TryGetAlt {
	my ($eh, $app, $tmpl_ref) = @_;
	my $jscode = <<END_OF_JS;
<script type="text/javascript">
function getAltText () {
	var str=getSelected(document.getElementById('text'));
	if ((str.indexOf('<') != -1) || (str.indexOf('>') != -1)) {
		return "";
	}
	return str;
}
</script>
END_OF_JS
	$$tmpl_ref =~ s/<\/head>/$jscode<\/head>/;
1;
}

sub _AddParam {
	my ($eh, $app, $tmpl_ref , $param_ref, $tmpl) = @_;
	my $url = $param_ref->{url};
	my $is_image = $param_ref->{is_image};
	my $width = $param_ref->{width};
	my $height = $param_ref->{height};
	my $search = $app->param;
	my $blogId = $search->param('blog_id');
	my $alt = $search->param('altField');
	$alt =~s/\n//g;
	my $qui = $search->param('quickInsertFlag');
	my $old = "&\' + mode\;";
	my $new = "&\' + mode"." + \'&altField=".$alt.'\';';
	$old = quotemeta($old);
	$$tmpl_ref =~ s/$old/$new/g;
1;
}

sub _AddHidden {
	my ($eh, $app, $tmpl_ref) = @_;
	my $search = $app->param;
	my $alt = $search->param('altField');
	$alt =~s/\n//g;
	my $old = '<h2>';
	my $new = '<input type="hidden" name="altField" value="'.$alt.'" />'."\n\n".$old;
	$old = quotemeta($old);
	$$tmpl_ref =~ s/$old/$new/g;
1;
}

sub checktime {
	my($hor, $min, $sec, $sep) = @_;
	if (&time_exists($hor, $min, $sec)) {
		return $hor.$kan_j.$min.$kan_h.$sec.$kan_b;
	} else {
		return $hor.$sep.$min.$sep.$sec;
	}
}

sub time_exists {
	my($hor, $min, $sec) = @_;
	if (($hor < 25) && ($min < 61) && ($sec < 61)) {
		return 1;
	} else {
		return 0;
	}
}

sub checkdate {
	my($yer, $mon, $day, $sep) = @_;
	if (&day_exists($yer, $mon, $day)) {
		return $yer.$kan_y.$mon.$kan_m.$day.$kan_d;
	} else {
		return $yer.$sep.$mon.$sep.$day;
	}
}

sub day_exists {
	my($year, $month, $day) = @_;
	my(@mlast) = (31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31);
	if ($month < 1 || 12 < $month) {
		return 0;
	}
	if ($month == 2) {
		if ((($year % 4 == 0) && ($year % 100 != 0)) || ($year % 400 == 0)) {
			$mlast[1]++;
		}
	}
	if ($day < 1 || $mlast[$month-1] < $day) {
		return 0;
	}
	return 1;
}

sub Ja {
    return <<'END';
+utf8::InHiragana
+utf8::InKatakana
+utf8::Han
END
}

sub monthToInt {
	my($strYear,$strMonth,$strDay,$week,$yStr,$mStr,$dStr)=@_;
	if ($strMonth eq "Jan") {return "$strYear$yStr"."1$mStr$strDay".$dStr};
	if ($strMonth eq "Feb") {return "$strYear$yStr"."2$mStr$strDay".$dStr};
	if ($strMonth eq "Mar") {return "$strYear$yStr"."3$mStr$strDay".$dStr};
	if ($strMonth eq "Apr") {return "$strYear$yStr"."4$mStr$strDay".$dStr};
	if ($strMonth eq "May") {return "$strYear$yStr"."5$mStr$strDay".$dStr};
	if ($strMonth eq "Jun") {return "$strYear$yStr"."6$mStr$strDay".$dStr};
	if ($strMonth eq "Jul") {return "$strYear$yStr"."7$mStr$strDay".$dStr};
	if ($strMonth eq "Aug") {return "$strYear$yStr"."8$mStr$strDay".$dStr};
	if ($strMonth eq "Sep") {return "$strYear$yStr"."9$mStr$strDay".$dStr};
	if ($strMonth eq "Oct") {return "$strYear$yStr"."10$mStr$strDay".$dStr};
	if ($strMonth eq "Nov") {return "$strYear$yStr"."11$mStr$strDay".$dStr};
	if ($strMonth eq "Dec") {return "$strYear$yStr"."12$mStr$strDay".$dStr};
	return "$week, $strDay $strMonth $strYear";
}
1;